# SPDX-FileCopyrightText: 2022 Tim Cocks for Adafruit Industries
# SPDX-FileCopyrightText: 2022 Matt Land
#
# SPDX-License-Identifier: MIT

"""
`adafruit_imageload.tilegrid_inflator`
====================================================

Use a 3x3 spritesheet to inflate a larger grid of tiles, duplicating the center rows and
columns as many times as needed to reach a target size.

* Author(s): Tim Cocks, Matt Land

"""
import displayio
import adafruit_imageload

try:
    from typing import Tuple, Optional, List, Union
    from displayio import Palette, Bitmap, OnDiskBitmap, TileGrid
except ImportError:
    pass

__version__ = "1.16.8"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_ImageLoad.git"


def inflate_tilegrid(
    bmp_path: str = None,
    target_size: Tuple[int, int] = (3, 3),
    tile_size: List[int] = None,
    transparent_index: Optional[Union[tuple, int]] = None,
    bmp_obj: Optional[OnDiskBitmap] = None,
    bmp_palette: Optional[Palette] = None,
) -> TileGrid:
    """
    inflate a TileGrid of ``target_size`` in tiles from a 3x3 spritesheet by duplicating
    the center rows and columns.

    :param Optional[str] bmp_path: filepath to the 3x3 spritesheet bitmap file
    :param Optional[tuple] target_size: desired size in tiles (target_width, target_height)
    :param Optional[tuple] tile_size: size of the tiles in the 3x3 spritesheet. If
      None is used it will equally divide the width and height of the Bitmap by 3.
    :param Optional[Union[tuple, int]] transparent_index: a single index within the palette to
      make transparent, or a tuple of multiple indexes to make transparent
    :param Optional[OnDiskBitmap] bmp_obj: Already loaded 3x3 spritesheet in an OnDiskBitmap
    :param Optional[Palette] bmp_palette: Already loaded spritesheet Palette
    """

    # pylint: disable=too-many-arguments, too-many-locals, too-many-branches

    if bmp_path is None and (bmp_obj is None and bmp_palette is None):
        raise AttributeError("Must pass either bmp_path or bmp_obj and bmp_palette")

    image: Bitmap
    palette: Palette
    if bmp_path is not None:
        image, palette = adafruit_imageload.load(bmp_path)  # type: ignore[assignment]
    else:
        image = bmp_obj  # type: ignore[assignment]
        palette = bmp_palette  # type: ignore[assignment]

    if transparent_index is not None:
        if isinstance(transparent_index, tuple):
            for index in transparent_index:
                palette.make_transparent(index)
        elif isinstance(transparent_index, int):
            palette.make_transparent(transparent_index)

    if tile_size is None:
        tile_width = image.width // 3
        tile_height = image.height // 3
    else:
        tile_width = tile_size[0]
        tile_height = tile_size[1]

    target_width = target_size[0]
    target_height = target_size[1]

    tile_grid = displayio.TileGrid(
        image,
        pixel_shader=palette,
        height=target_height,
        width=target_width,
        tile_width=tile_width,
        tile_height=tile_height,
    )

    # corners
    tile_grid[0, 0] = 0  # upper left
    tile_grid[tile_grid.width - 1, 0] = 2  # upper right
    tile_grid[0, tile_grid.height - 1] = 6  # lower left
    tile_grid[tile_grid.width - 1, tile_grid.height - 1] = 8  # lower right

    for x in range(target_size[0] - 2):
        tile_grid[x + 1, 0] = 1
        tile_grid[x + 1, tile_grid.height - 1] = 7

    for y in range(target_size[1] - 2):
        tile_grid[0, y + 1] = 3
        tile_grid[tile_grid.width - 1, y + 1] = 5

    for y in range(target_size[1] - 2):
        for x in range(target_size[0] - 2):
            tile_grid[x + 1, y + 1] = 4

    return tile_grid



# SPDX-FileCopyrightText: 2018 Scott Shawcroft for Adafruit Industries
# SPDX-FileCopyrightText: 2022 Matt Land
# SPDX-FileCopyrightText: Brooke Storm
# SPDX-FileCopyrightText: Sam McGahan
#
# SPDX-License-Identifier: MIT

"""
`adafruit_imageload.pnm.pgm`
====================================================

Load pixel values (indices or colors) into a bitmap and colors into a palette.

* Author(s): Matt Land, Brooke Storm, Sam McGahan

"""
# pylint: disable=import-outside-toplevel
try:
    from typing import Tuple, Optional, Set, List
    from io import BufferedReader
    from displayio import Palette, Bitmap
    from ...displayio_types import PaletteConstructor, BitmapConstructor
except ImportError:
    pass


def load(
    file: BufferedReader,
    magic_number: bytes,
    header: List[int],
    *,
    bitmap: BitmapConstructor = None,
    palette: PaletteConstructor = None
) -> Tuple[Optional[Bitmap], Optional[Palette]]:
    """
    Perform the load of Netpbm greyscale images (P2, P5)
    """
    if header[2] > 256:
        raise NotImplementedError("16 bit files are not supported")
    width = header[0]
    height = header[1]

    if magic_number == b"P2":  # To handle ascii PGM files.
        from . import ascii as pgm_ascii

        return pgm_ascii.load(file, width, height, bitmap=bitmap, palette=palette)

    if magic_number == b"P5":  # To handle binary PGM files.
        from . import binary

        return binary.load(file, width, height, bitmap=bitmap, palette=palette)

    raise NotImplementedError("Was not able to send image")



# SPDX-FileCopyrightText: 2018 Scott Shawcroft for Adafruit Industries
# SPDX-FileCopyrightText: 2022 Matt Land
# SPDX-FileCopyrightText: Brooke Storm
# SPDX-FileCopyrightText: Sam McGahan
#
# SPDX-License-Identifier: MIT

"""
`adafruit_imageload.pnm.ppm_binary`
====================================================

Load pixel values (indices or colors) into a bitmap and for a binary ppm,
return None for pallet.

* Author(s):  Matt Land, Brooke Storm, Sam McGahan

"""
try:
    from typing import Tuple, Optional, List, Set
    from io import BufferedReader
    from displayio import Palette, Bitmap
    from ..displayio_types import PaletteConstructor, BitmapConstructor
except ImportError:
    pass

__version__ = "1.16.8"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_ImageLoad.git"


def load(
    file: BufferedReader,
    width: int,
    height: int,
    bitmap: BitmapConstructor = None,
    palette: PaletteConstructor = None,
) -> Tuple[Optional[Bitmap], Optional[Palette]]:
    """Load pixel values (indices or colors) into a bitmap and for a binary
    ppm, return None for pallet."""

    data_start = file.tell()
    palette_colors = set()  # type: Set[Tuple[int, int, int]]
    line_size = width * 3

    for y in range(height):
        data_line = iter(bytes(file.read(line_size)))
        for red in data_line:
            # red, green, blue
            palette_colors.add((red, next(data_line), next(data_line)))

    if palette:
        palette = palette(len(palette_colors))  # type: Palette
        for counter, color in enumerate(palette_colors):
            palette[counter] = bytes(color)
    if bitmap:
        bitmap = bitmap(width, height, len(palette_colors))  # type: Bitmap
        file.seek(data_start)
        palette_colors = list(palette_colors)  # type: List[Tuple[int, int, int]]
        for y in range(height):
            x = 0
            data_line = iter(bytes(file.read(line_size)))
            for red in data_line:
                # red, green, blue
                bitmap[x, y] = palette_colors.index(
                    (red, next(data_line), next(data_line))
                )
                x += 1

    return bitmap, palette



# SPDX-FileCopyrightText: 2022 Tim Cocks for Adafruit Industries
# SPDX-License-Identifier: MIT

import board
import displayio
import adafruit_imageload
from adafruit_imageload.tilegrid_inflator import inflate_tilegrid

image, palette = adafruit_imageload.load("images/castle_spritesheet.bmp")
tile_grid = inflate_tilegrid(bmp_obj=image, bmp_palette=palette, target_size=(10, 8))

group = displayio.Group()
group.append(tile_grid)
board.DISPLAY.show(group)

while True:
    pass



# SPDX-FileCopyrightText: 2022 Radomir Dopieralski
#
# SPDX-License-Identifier: MIT

"""
`adafruit_imageload.png`
====================================================

Load pixel values (indices or colors) into a bitmap and colors into a palette
from a PNG file.

* Author(s): Radomir Dopieralski

"""

import struct
import zlib


__version__ = "0.0.0-auto.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_ImageLoad.git"


def load(
    file, *, bitmap, palette=None
):  # pylint: disable=too-many-locals,too-many-branches
    """Loads a PNG image from the open ``file``.

    Returns tuple of bitmap object and palette object.

    :param file: The *.png file being loaded
    :param object bitmap: Type to store bitmap data. Must have API similar to
      `displayio.Bitmap`.
    :param object palette: Type to store the palette. Must have API similar to
      `displayio.Palette`. Will be skipped if None"""
    header = file.read(8)
    if header != b"\x89PNG\r\n\x1a\n":
        raise ValueError("Not a PNG file")
    del header
    data = bytearray()
    pal = None
    mode = None
    depth = None
    while True:
        size, chunk = struct.unpack(">I4s", file.read(8))
        if chunk == b"IHDR":
            (
                width,
                height,
                depth,
                mode,
                compression,
                filters,
                interlaced,
            ) = struct.unpack(">IIBBBBB", file.read(13))
            if interlaced:
                raise NotImplementedError("Interlaced images unsupported")
            # compression and filters must be 0 with current spec
            assert compression == 0
            assert filters == 0
        elif chunk == b"PLTE":
            if palette is None:
                file.seek(size, 1)
            else:
                if mode != 3:
                    raise NotImplementedError("Palette in non-indexed image")
                pal_size = size // 3
                pal = palette(pal_size)
                for i in range(pal_size):
                    pal[i] = file.read(3)
        elif chunk == b"IDAT":
            data.extend(file.read(size))
        elif chunk == b"IEND":
            break
        else:
            file.seek(size, 1)  # skip unknown chunks
        file.seek(4, 1)  # skip CRC
    data = zlib.decompress(data)
    bmp = bitmap(width, height, 1 << depth)
    scanline = (width * depth + 7) // 8
    mem = memoryview(bmp)
    for y in range(height):
        dst = y * scanline
        src = y * (scanline + 1) + 1
        filter_ = data[src - 1]
        if filter_ == 0:
            mem[dst : dst + scanline] = data[src : src + scanline]
        else:
            raise NotImplementedError("Filters not supported")
    return bmp, pal
